-- Daily Global Check
-- by Jadya
-- EU-Well of Eternity
 
--[[
 - values
  - questID    : string
  - type       : int
  - prefix     : string
  - zone       : table  - [1] = string (zone name), [2] = {mapID, x, y, nil, ... }
  - name       : string
  - suffix     : string
  - completion : string
]]

local max, lower, tsort, format = math.max, string.lower, table.sort, string.format

local addonName, addonTable = ...

local AddColor = addonTable.AddColor

local L = addonTable.L
local C = addonTable.C
local ng = NyxGUI(C.NyxGUIVersion)

local constructors = DailyGlobalCheck.Constructors
local lists = DailyGlobalCheck.ListFunctions
local tools = DailyGlobalCheck.Tools
local modes = DailyGlobalCheck.ModeFunctions
local quest = DailyGlobalCheck.QuestFunctions
local maps  = DailyGlobalCheck.MapFunctions
local help  = DailyGlobalCheck.HelpFunctions

local questID_tab_groups = { ["achicriteria"] = "achievement", 
                             ["boss"]         = "instance" }

local db_spells = {}
local search_filters, inits, updates = {}, {}, {}
local db_zone
local qID, qData
local preview

local function questID2Input(questID, key)
 if key == "multipleor" then
  return questID:gsub("multipleor#", ""):gsub("_", " or ")
 elseif key == "multipleand" then
  return questID:gsub("multipleand#", ""):gsub("_", " and ")
 end
 return questID
end

local function input2QuestID(questID)
 questID = lower(tostring(questID):gsub(" ", ""))
 if strfind(questID, "or") then
  return "multipleor#"..questID:gsub("or", "_")
 elseif strfind(questID, "and") then
  return "multipleand#"..questID:gsub("and", "_")
 end
 return questID
end

local function questID_change(old, new)
 local questsdata = DGC_CustomLists.QuestsData

 new = input2QuestID(new)

 if not old or not new or new == "" then 
  return
 elseif old == new then
  return true
 elseif questsdata[new] then
  print(L["bf_err2"])
  return
 end
 
 if questsdata[old] then
  questsdata[new] = questsdata[old]
  questsdata[old] = nil
  questsdata[new].questID = new
 end
 
 for _,list in pairs(DGC_CustomLists.Lists) do
  for _,page in pairs(list.Order) do
   for _,group in pairs(page) do
    for k,v in pairs(group) do
	 if k > 1 and v == old then
	  group[k] = new
	 end
	end
   end
  end
 end
 
 return true
end

local function questID_updatetab(t, qID)
 
 local btn = DGC_EQ_Tabbutton_quest
 local result = ""
 
 if t then
  btn = _G["DGC_EQ_Tabbutton_"..(questID_tab_groups[t] or t)] or DGC_EQ_Tabbutton_quest
 end

 if not btn then return end

 -- fill widgets
 if btn == DGC_EQ_Tabbutton_quest then
  result = questID2Input(qID, t)
  DGC_EQQ_questID:SetText(result)
 elseif btn == DGC_EQ_Tabbutton_spell then
  DGC_EQQ_spellID:SetText(qID and qID:gsub("#", ""):gsub(t or "", "") or "")
 end

 btn:Click("LeftButton")
 return result
end

local function fitText(w, s, col)
 local width = w:GetWidth() / 9

 s = tostring(s or "") 

 local l = #s
 
 s = addonTable.cutString(s):sub(1, width)
 
 w:SetText(AddColor(s..((#s < l) and ".." or ""), col))
end

local function data2widgets()
 
 local c = DailyGlobalCheck_Options.Colors

 -- questID
  local qt = quest:GetSpecialType(qID)
  local v = questID_updatetab(qt, qID)
  preview["QuestID"].value = v
  fitText(preview["QuestID"], v)
 -- quest type
  qt = C.questtype_strings[qData[7]] or ""
  local widget = _G["DGC_EQQT_"..qt]
  if widget then
   widget:Click()--SetChecked(true)
  end
  preview["Type"].value = qData[7]
  fitText(preview["Type"], qt)
 -- prefix
  DGC_EQP_prefix:SetText(qData[3])
  preview["Prefix"].value = qData[3]
  fitText(preview["Prefix"], qData[3], c[3])
 -- zone
 fitText(preview["Zone"], qData[1], c[1])
 if qData[5] then
  local m = qData[5]
  DGC_EQ_Tabbutton_zoneselect:Click("LeftButton")
  if DGC_EQZC_list then
   for k,v in pairs(DGC_EQZC_list.list) do
    if v == m[1] then
     DGC_EQZC_list.sel_index = k
    end
   end
   local x, y = floor(m[2]), floor(m[3])
   DGC_EQZC_coordX:SetText(x)
   DGC_EQZC_coordY:SetText(y)
   fitText(preview["Coords"], x.." "..y, c[5])
   updates.zoneselect()
  end
 else -- simple text
  DGC_EQ_Tabbutton_zonesimple:Click("LeftButton")
  preview["Coords"]:SetText("")
  updates.zonesimple()
 end
 -- name
  DGC_EQP_name:SetText(qData[2])
  preview["Name"].value = qData[2]
  fitText(preview["Name"], qData[2], c[2])
 -- suffix
  DGC_EQP_suffix:SetText(qData[4])
  preview["Suffix"].value = qData[4]
  fitText(preview["Suffix"], qData[4], c[4])
end

local function widgets2data()
 
 local qID_result = questID_change(qID, preview["QuestID"].value)
 
 if not qID_result then
  print("|cff11DDFFDaily Global Check|r - QuestID requires changes")
  return
 end

 qData[1] = preview["Zone"].value[1]
 qData[2] = preview["Name"].value
 qData[3] = preview["Prefix"].value
 qData[4] = preview["Suffix"].value
 qData[5] = preview["Zone"].value[2]
 qData[6] = nil -- mapdata behavior nyi
 qData[7] = preview["Type"].value
 return true
end

-- Generic Editbox
local function editbox_textchanged(self)
 local s = self:GetText()

 fitText(preview[self.tag], s, self.i and DailyGlobalCheck_Options.Colors[self.i])
 
 if self == DGC_EQQ_questID then
  s = tonumber(s) or s
 elseif self == DGC_EQQ_spellID then
  s = "spell#"..s
  DGC_EQQ_spellID:update_info()
 end
 
 preview[self.tag].value = s
end
--
-- internal tab updates
-- questID
function updates.quest()
 editbox_textchanged(DGC_EQQ_questID)
 preview["QuestID"].label:SetText(L["b_questid"])
 preview["Type"].label:SetText("Type")
 preview["Type"]:Enable()
end

function updates.spell()
 editbox_textchanged(DGC_EQQ_spellID)
 preview["QuestID"].label:SetText(L["spell_id"])
 preview["Type"].label:SetText(AddColor("Type", "GRAY"))
 preview["Type"]:Disable()
end
-- Zone
function updates.zonesimple()
 local zone = DGC_EQZS_zone:GetText()
 fitText(preview["Zone"], zone, DailyGlobalCheck_Options.Colors[1])
 preview["Zone"].value[1] = zone
 preview["Zone"].value[2] = nil
 preview["Zone"].value[3] = nil
 preview["Coords"]:SetText("")
end

function updates.zoneselect()
 local zone, x, y = "", tonumber(DGC_EQZC_coordX:GetText()) or 0, tonumber(DGC_EQZC_coordY:GetText()) or 0
 local data

 if DGC_EQZC_list.sel_index then
  zone = db_zone[DGC_EQZC_list.list[DGC_EQZC_list.sel_index]]
  -- multiple icons to be implemented
  data = { DGC_EQZC_list.list[DGC_EQZC_list.sel_index], x, y }
 end

 -- zone name
 fitText(preview["Zone"], zone, DailyGlobalCheck_Options.Colors[1])
 preview["Zone"].value[1] = zone
 -- coords
 preview["Zone"].value[2] = data
 --preview["Zone"].value[3] = nil -- mapdata behavior not yet implemented
 fitText(preview["Coords"], floor(x).." "..floor(y), DailyGlobalCheck_Options.Colors[5])
end
--
-- questID
function inits.quest(tab)
 local f = ng:New(addonName, "Label", "DGC_EQQ_questID_infolabel", tab)
 f:SetPoint("BOTTOMLEFT", tab, "BOTTOMLEFT", 10, 20)
 f:SetPoint("BOTTOMRIGHT", tab, "BOTTOMRIGHT", -10, 0)
 f:SetText(format(L["edit_questID_info"], "|cff00FF00or|r", "|cff00FF00and|r"))
 f:SetJustifyH("LEFT")

 f = ng:New(addonName, "Editbox", "DGC_EQQ_questID", tab)
 f:SetPoint("TOPLEFT", tab, "TOPLEFT", 10, -20)
 f:SetWidth(400)
 f:SetTitle(L["b_questid"])
 f.tag = "QuestID"
 f:SetScript("OnTextChanged", editbox_textchanged)
end

function inits.spell(tab)
 local f = ng:New(addonName, "Label", "DGC_EQQ_spellID_infolabel", tab)
 f:SetPoint("BOTTOMLEFT", tab, "BOTTOMLEFT", 10, 10)
 f:SetPoint("BOTTOMRIGHT", tab, "BOTTOM", 0, 0)
 f:SetJustifyH("LEFT")
 f:SetText(L["edit_spell_info"])

 f = ng:New(addonName, "Editbox", "DGC_EQQ_spellID", tab)
 f:SetPoint("TOPLEFT", tab, "TOPLEFT", 10, -20)
 f:SetWidth(150)
 f:SetTitle(L["spell_id"])
 f.tag = "QuestID"
 f:SetScript("OnTextChanged", editbox_textchanged) 
 
 f.info_label = ng:New(addonName, "Label", nil, tab)
 f.info_label:SetPoint("TOPLEFT", f, "BOTTOMLEFT", 0, -65)
 f.info_label:SetPoint("BOTTOMRIGHT", f, "BOTTOMRIGHT", 0, -75)
 f.info_icon = f:CreateTexture()
 f.info_icon:SetPoint("TOP", f, "BOTTOM", 0, -10)
 f.info_icon:SetSize(50, 50)
 f.update_info = function(self)
                  local id = tonumber(self:GetText())
				  if id then
                   local name, _, icon = GetSpellInfo(id)
				   self.info_label:SetText(name)
				   self.info_icon:SetTexture(icon)
				  else
				   self.info_label:SetText("")
				   self.info_icon:SetTexture()
				  end
                 end
 
 local function init_search_section()
  local f = ng:New(addonName, "Editbox", "DGC_EQspell_filter", tab)
  f:SetWidth(240)
  f:SetPoint("TOPRIGHT", tab, "TOPRIGHT", -40, -20)
  f:SetTitle(SEARCH)
  f:SetScript("OnTextChanged", function() 
                    local s = lower(DGC_EQspell_filter:GetText())
                    wipe(DGC_EQspell_list.list)
					
					if not s or s == "" then 
					 DGC_EQspell_list:update()
					 DGC_EQspell_list.warning_label:SetText("")
					 return
					end

					local c = s:sub(1,1)
					
					local counter = 1
                    for k,v in pairs(db_spells) do
                     if lower(v):find(s) then
					  local i = #DGC_EQspell_list.list + 1
                      DGC_EQspell_list.list[i] = k
					  counter = counter + 1
					  if counter == 100 then break end
                     end
                    end
					
					if counter >= 100 then
					 DGC_EQspell_list.warning_label:SetText(format("Displaying the first %s results", AddColor(counter, "LIGHT_GREEN")))
					else
					 DGC_EQspell_list.warning_label:SetText("")
					end

                    tsort(DGC_EQspell_list.list, sortfunc)
					DGC_EQspell_list:update()
                   end)
  search_filters[#search_filters + 1] = f
				   
  f = ng:New(addonName, "Listbox", "DGC_EQspell_list", tab)
 
  f.warning_label = ng:New(addonName, "Label", nil, tab)
  f.warning_label:SetPoint("TOP", DGC_EQspell_list, "BOTTOM", 0, -10)
 
  f.list = {}
  f.update_button = function(self, list, index)
                     self.left_text:SetText(db_spells[list[index]])
                    end

  f:SetSize(220, 200)
  f:Initialize("DGCSimpleListButton")
  f:SetPoint("TOPLEFT", DGC_EQspell_filter, "BOTTOMLEFT", 0, -10)
  f:SetPoint("BOTTOMRIGHT", DGC_EQspell_filter, "BOTTOMRIGHT", 0, -175)

  for _,v in pairs(f.buttons) do
   ng:AddToTheme(addonName, nil, "Listbutton", v)
   v.func = function(self)
             DGC_EQQ_spellID:SetText(DGC_EQspell_list.list[self.index])
			end
  end
 end

 local function init_spell_database()
  local i = 0

  local progressbar = ng:New(addonName, "Progressbar", nil, tab)
  progressbar:SetWidth(175)
  progressbar:SetPoint("CENTER", tab, "CENTER", 0, 40)
  progressbar:SetText(format(L["generating_database"], lower(SPELLS)))
  
  C_Timer.NewTicker(0.025, function(self)
                            for k = 1, 750 do
 						     i = i + 1
                             local name = GetSpellInfo(i)
 						     if name then
							  name = lower(name)
 						      db_spells[i] = name
 						     end
 						    end
							progressbar:SetValue(i / 250000 * 100)
 						    if i > 250000 then
 						     self:Cancel()
							 init_search_section()
							 progressbar:Hide()
 						    end
 						   end)
 end

 f = ng:New(addonName, "Button", nil, tab)
 f:SetSize(150, 40)
 f:SetPoint("CENTER", tab, "CENTER", 0, 40)
 f:SetText(L["search_by_name"])
 f:SetScript("OnClick", function(self) self:Hide() init_spell_database() end)
end

-- zone 1 - Simple Text
function inits.zonesimple(tab)
 local f = ng:New(addonName, "Editbox", "DGC_EQZS_zone", tab)
 f:SetWidth(220)
 f:SetPoint("TOPLEFT", tab, "TOPLEFT", 10, -20)
 f:SetTitle(ZONE)
 f.tag = "Zone"
 f.i = 1
 f:SetScript("OnTextChanged", updates.zonesimple)
end
--

-- zone 2 - Select Zone
local function init_zone_database()
 db_zone = {}
 
 local function add(k)
  if GetMapNameByID(k) then
   db_zone[k] = GetMapNameByID(k)
  end
 end

 -- draenor
 for _,v in pairs({978,941,976,949,971,950,947,948,1009,946,945,1011}) do
  add(v)
 end
 -- pandaria
 for _,v in pairs({858,929,928,857,809,905,903,806,873,808,951,810,811,807}) do
  add(v)
 end
 -- northrend
 for _,v in pairs({486,510,504,488,490,491,541,492,493,495,501,496}) do
  add(v)
 end
 -- outlands
 for _,v in pairs({475,465,477,479,473,481,478,467}) do
  add(v)
 end
 -- kalimdor
 for _,v in pairs({772,894,43,181,464,476,890,42,381,101,4,141,891,182,121,795,241,606,9,11,321,888,261,607,81,161,41,471,61,362,720,201,889,281}) do
  add(v)
 end
 -- eastern
 for _,v in pairs({614,16,17,19,29,866,32,892,27,34,23,30,462,463,545,611,24,341,499,610,35,895,37,864,36,684,685,28,615,480,21,301,689,893,38,673,26,502,20,708,709,700,382,613,22,39,40}) do
  add(v)
 end
end

local function sortfunc(a, b)
 return db_zone[a] < db_zone[b]
end

local function zoneselect_listbox_update_pre(self)
 local s = lower(DGC_EQZC_filter:GetText())
 wipe(self.list)
 
 for k,v in pairs(db_zone) do
  if lower(v):find(s) then
   self.list[#self.list + 1] = k
  end
 end
 
 tsort(self.list, sortfunc)
end

local function zoneselect_listbox_update_button(self, list, index)
 self.left_text:SetText(db_zone[list[index]])
 self.right_text:SetText("#"..list[index])
end

local function zoneselect_listbutton_click(self)
 updates.zoneselect()
end

function inits.zoneselect(tab)
 init_zone_database()

 local f = ng:New(addonName, "Editbox", "DGC_EQZC_filter", tab)
 f:SetWidth(220)
 f:SetPoint("TOPLEFT", tab, "TOPLEFT", 10, -20)
 f:SetTitle(SEARCH)
 f:SetScript("OnTextChanged", function() DGC_EQZC_list:update() end)
 
 f = ng:New(addonName, "Listbox", "DGC_EQZC_list", tab)
 f.list       = {}
 f.update_pre = zoneselect_listbox_update_pre
 f.update_button = zoneselect_listbox_update_button
 
 f:SetSize(220, 200)
 f:Initialize("DGCSimpleListButton")
 f:SetPoint("TOPLEFT", DGC_EQZC_filter, "BOTTOMLEFT", 0, -10)
 f:SetPoint("BOTTOMRIGHT", DGC_EQZC_filter, "BOTTOMRIGHT", 0, -195)

 for _,v in pairs(f.buttons) do
  ng:AddToTheme(addonName, nil, "Listbutton", v)
  v.func = zoneselect_listbutton_click
 end
 
 local g1 = ng:New(addonName, "Groupbox", nil, tab)
 g1:SetPoint("TOPLEFT", DGC_EQZC_list, "TOPRIGHT", 40, 0)
 g1:SetSize(140, 40)
 g1:SetText(L["coordinates"])
 
 f = ng:New(addonName, "Editbox", "DGC_EQZC_coordX", tab)
 f:SetWidth(60)
 f:SetPoint("TOPLEFT", g1, "TOPLEFT", 5, -15)
 f:SetTitle("X")
 f:SetScript("OnTextChanged", function() updates.zoneselect() end)

 f = ng:New(addonName, "Editbox", "DGC_EQZC_coordY", tab)
 f:SetWidth(60)
 f:SetPoint("LEFT", DGC_EQZC_coordX, "RIGHT", 10, 0)
 f:SetTitle("Y")
 f:SetScript("OnTextChanged", function() updates.zoneselect() end)
end
--

local function btnok_click()
 if widgets2data() then
  modes:Set(C.MODE_EDIT)
 else
  -- in caso di questID uguali bloccare tutto e cerchiare in rosso preview["QuestID"]
 end
end

local function btncancel_click()
 modes:Set(C.MODE_EDIT)
end

local bg = { bgFile = "Interface\\Buttons\\GreyscaleRamp64", 
             edgeFile = "Interface\\Buttons\\WHITE8X8", 
	         edgeSize = 1
		   }

function tools.setup_edit_quest_panel(p)

 local g1 = ng:New(addonName, "Groupbox", nil, p)
 g1:SetPoint("TOPLEFT")
 g1:SetPoint("BOTTOMRIGHT")
 
 local g2 = ng:New(addonName, "Groupbox", nil, g1)
 g2:SetPoint("TOPLEFT", g1, "TOPLEFT", 10, -20)
 g2:SetPoint("BOTTOMRIGHT", g1, "TOPRIGHT", -10, -70)

 local editgroup = ng:New(addonName, "Groupbox", nil, g2)
 editgroup:SetPoint("TOPLEFT", g2, "BOTTOMLEFT", 0, -10)
 editgroup:SetPoint("BOTTOMRIGHT", p, "BOTTOMRIGHT", -10, 40)

 local anchor = 10
 local function createPreviewButton(key, w, tabindex)
  local btn = ng:New(addonName, "Tabbutton", "DGCEQ_"..key, g2, nil, tabindex)
  local label = ng:New(addonName, "Label", nil, btn)
  label:SetText(key)
  label:SetPoint("BOTTOM", btn, "TOP")
  btn.label = label
  btn:ClearAllPoints()
  btn:SetPoint("LEFT", g2, "LEFT", anchor, 0)
  anchor = anchor + w
  btn:SetSize(w - 5, 15)
  btn.value = ""
  preview[key] = btn
 end

 preview = {}
 createPreviewButton("QuestID"   , 75)
 createPreviewButton("Type"      , 25 )
 createPreviewButton("Prefix"    , 75 )
 createPreviewButton("Zone"      , 100)
 createPreviewButton("Name"      , 150, 3)
 createPreviewButton("Suffix"    , 75 , 3)
 createPreviewButton("Coords"    , 75 , 4)
 createPreviewButton("Completion", 75)

 preview["Zone"].value = {}

 ------------ questID group ------------
 g1 = ng:New(addonName, "Groupbox", nil, preview["QuestID"].tab)
 g1:SetPoint("TOPLEFT", editgroup, "TOPLEFT", 10, -40)
 g1:SetPoint("BOTTOMRIGHT", editgroup, "BOTTOMRIGHT", -10, 10)
 
 local function createTab(t, text, w, group)
  local result = ng:New(addonName, "Tabbutton", "DGC_EQ_Tabbutton_"..t, group)
  result:SetWidth(w)
  result:SetText(text)
  if inits[t] then
   inits[t](result.tab)
  end
  if updates[t] then
   result.tab:SetScript("OnShow", updates[t])
  end
  result.tab:Hide()
  return result.tab
 end

 -- quest
 createTab("quest", L["b_questid"], 100, g1)
 createTab("spell", STAT_CATEGORY_SPELL, 100, g1)

 -- instance
 --tab = createTab("instance", "Instance", 100, g1)
 --createTab("boss", "Instance Boss", 100, g1)
 --createTab("achievement", "Achievement", 100, g1)
 --createTab("achicriteria", "Achi. Criteria", 100, g1)
 --createTab("currency", "Currency", 100, g1)

 ------------------------

 ------------ quest type group ------------

 local function chk_questtype_onclick(self)
  fitText(preview["Type"], C.questtype_strings[self.tag] or "D")
  preview["Type"].value = self.tag
 end
 
 local f = ng:New(addonName, "Radiobutton", "DGC_EQQT_Q", preview["Type"].tab)
 f:SetPoint("TOPLEFT", editgroup, "TOPLEFT", 10, -10)
 f:SetText("Quest")
 f.tag = 1
 f.func = chk_questtype_onclick
 
 f = ng:New(addonName, "Radiobutton", "DGC_EQQT_D", preview["Type"].tab)
 f:SetPoint("TOPLEFT", editgroup, "TOPLEFT", 10, -30)
 f:SetText("Daily")
 f.func = chk_questtype_onclick
 
 f = ng:New(addonName, "Radiobutton", "DGC_EQQT_W", preview["Type"].tab)
 f:SetPoint("TOPLEFT", editgroup, "TOPLEFT", 10, -50)
 f:SetText("Weekly")
 f.tag = 3
 f.func = chk_questtype_onclick
 
 f = ng:New(addonName, "Radiobutton", "DGC_EQQT_R", preview["Type"].tab)
 f:SetPoint("TOPLEFT", editgroup, "TOPLEFT", 10, -70)
 f:SetText("Repeatable")
 f.tag = 13
 f.func = chk_questtype_onclick
 
 f = ng:New(addonName, "Radiobutton", "DGC_EQQT_Qa", preview["Type"].tab)
 f:SetPoint("TOPLEFT", editgroup, "TOPLEFT", 10, -100)
 f:SetText("Quest (Account-Wide)")
 f.tag = 14
 f.func = chk_questtype_onclick
 
 f = ng:New(addonName, "Radiobutton", "DGC_EQQT_Da", preview["Type"].tab)
 f:SetPoint("TOPLEFT", editgroup, "TOPLEFT", 10, -120)
 f:SetText("Daily (Account-Wide)")
 f.tag = 15
 f.func = chk_questtype_onclick
 
 f = ng:New(addonName, "Radiobutton", "DGC_EQQT_Wa", preview["Type"].tab)
 f:SetPoint("TOPLEFT", editgroup, "TOPLEFT", 10, -140)
 f:SetText("Weekly (Account-Wide)")
 f.tag = 16
 f.func = chk_questtype_onclick

 --[[f = ng:New(addonName, "Radiobutton", "DGC_EQQT_instance", editgroup)
 f:SetPoint("TOPLEFT", editgroup, "TOPLEFT", 10, -70)
 f:SetText("Instance")]]
 
 ------------------------

 ------------ prefix name and suffix ------------
 
 f = ng:New(addonName, "Editbox", "DGC_EQP_prefix", preview["Prefix"].tab)
 f:SetWidth(220)
 f:SetPoint("TOPLEFT", editgroup, "TOPLEFT", 10, -20)
 f:SetTitle(L["b_prefix"])
 f.tag = "Prefix"
 f.i = 3
 f:SetScript("OnTextChanged", editbox_textchanged)

 f = ng:New(addonName, "Editbox", "DGC_EQP_name", preview["Name"].tab)
 f:SetWidth(220)
 f:SetPoint("TOPLEFT", editgroup, "TOPLEFT", 10, -60)
 f:SetTitle(NAME)
 f.tag = "Name"
 f.i = 2
 f:SetScript("OnTextChanged", editbox_textchanged)

 f = ng:New(addonName, "Editbox", "DGC_EQP_suffix", preview["Suffix"].tab)
 f:SetWidth(220)
 f:SetPoint("TOPLEFT", editgroup, "TOPLEFT", 10, -100)
 f:SetTitle(L["b_suffix"])
 f.tag = "Suffix"
 f.i = 4
 f:SetScript("OnTextChanged", editbox_textchanged)
 
 ------------------------

 ------------ not available message ------------
 
 f = ng:New(addonName, "Label", "DGC_EQZ_tmplabel", preview["Completion"].tab)
 f:SetPoint("CENTER", editgroup, "CENTER")
 f:SetText("|TInterface\\Minimap\\Vehicle-HammerGold-1:32|t Not available at the moment")
  
 ------------------------

 -- zone
 g1 = ng:New(addonName, "Groupbox", nil, preview["Zone"].tab)
 g1:SetPoint("TOPLEFT", editgroup, "TOPLEFT", 10, -40)
 g1:SetPoint("BOTTOMRIGHT", editgroup, "BOTTOMRIGHT", -10, 10)

 -- tab 1 ( simple )
 createTab("zonesimple", LOCALE_TEXT_LABEL, 95, g1)

 -- tab 2 ( select zone )
 createTab("zoneselect", "Select Zone", 95, g1)
 --

 -- ok button
 f = ng:New(addonName, "Button", nil, p)
 f:SetPoint("BOTTOM", p, "BOTTOM", -40, 8)
 f:SetText(OKAY)
 f:SetScript("OnClick", btnok_click)
 f:SetWidth(75)
 
 -- cancel button
 f = ng:New(addonName, "Button", nil, p)
 f:SetPoint("BOTTOM", p, "BOTTOM", 40, 8)
 f:SetText(CANCEL)
 f:SetScript("OnClick", btncancel_click)
 f:SetWidth(75)

 -- help button
 f = CreateFrame("Button", "DGC_EQHelpButton", p, "MainHelpPlateButton")
 f:SetPoint("TOPLEFT", DGC_Mainframe, "TOPLEFT", -5, 5)
 f:SetScript("OnClick", function()
  local hp = addonTable.HelpPlates.EditQuest

  if not help:isShowing(hp) then
   help:Show(hp, DGC_Mainframe, DGC_EQHelpButton)
  else
   help:Hide()
  end
 end)
end

function tools:EditQuest(questID, data)

 if not data then return end
 
 qID = questID
 qData = data
 modes:Set(C.MODE_EDIT_QUEST)
 
 data2widgets()
 
 for _,v in pairs(search_filters) do
  v:SetText("")
 end
 
 DGCEQ_QuestID:Click("Leftbutton")
end







